<?php

namespace Kaizen_Coders\Url_Shortify\Admin\DB;

use Kaizen_Coders\Url_Shortify\Common\Utils;
use Kaizen_Coders\Url_Shortify\Helper;

class Links extends Base_DB {
	/**
	 * Table Name
	 *
	 * @since 1.0.0
	 * @var string
	 *
	 */
	public $table_name;
	/**
	 * Table Version
	 *
	 * @since 1.0.0
	 * @var string
	 *
	 */
	public $version;
	/**
	 * Primary key
	 *
	 * @since 1.0.0
	 * @var string
	 *
	 */
	public $primary_key;

	/**
	 * Initialize
	 *
	 * constructor.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		global $wpdb;

		parent::__construct();

		$this->table_name = $wpdb->prefix . 'kc_us_links';

		$this->version = '1.0';

		$this->primary_key = 'id';
	}

	/**
	 * Get columns and formats
	 *
	 * @since 1.0.0
	 */
	public function get_columns() {

		return array(
			'id'                => '%d',
			'name'              => '%s',
			'slug'              => '%s',
			'url'               => '%s',
			'description'       => '%s',
			'nofollow'          => '%d',
			'track_me'          => '%d',
			'sponsored'         => '%d',
			'params_forwarding' => '%d',
			'params_structure'  => '%s',
			'redirect_type'     => '%s',
			'status'            => '%d',
			'type'              => '%s',
			'type_id'           => '%d',
			'password'          => '%s',
			'expires_at'        => '%s',
			'cpt_id'            => '%d',
			'cpt_type'          => '%s',
			'rules'             => '%s',
			'created_at'        => '%s',
			'created_by_id'     => '%d',
			'updated_at'        => '%s',
			'updated_by_id'     => '%d',
		);
	}

	/**
	 * Get default column values
	 *
	 * @since 1.0.0
	 */
	public function get_column_defaults() {

		return array(
			'name'              => '',
			'slug'              => '',
			'description'       => '',
			'url'               => null,
			'nofollow'          => 0,
			'track_me'          => 1,
			'sponsored'         => 0,
			'params_forwarding' => 0,
			'params_structure'  => null,
			'redirect_type'     => 307,
			'status'            => 1,
			'type'              => 'direct',
			'type_id'           => null,
			'password'          => null,
			'expires_at'        => null,
			'cpt_id'            => null,
			'cpt_type'          => null,
			'rules'             => null,
			'created_at'        => Helper::get_current_date_time(),
			'created_by_id'     => null,
			'updated_at'        => null,
			'updated_by_id'     => null,
		);
	}

	/**
	 * Get link data based on link_id
	 *
	 * @param null $link_id
	 * @param string $output
	 *
	 * @return array|object|void|null
	 *
	 * @since 1.1.3
	 */
	public function get( $link_id = null, $output = ARRAY_A ) {

		if ( empty( $link_id ) ) {
			return [];
		}

		$link_data = parent::get( $link_id, $output );

		$groups = US()->db->links_groups->get_group_ids_by_link_ids( $link_id );

		$group_ids = Helper::get_data( $groups, $link_id, [] );

		$link_data['group_ids'] = $group_ids;

		return $link_data;
	}

	/**
	 * Get link by slug
	 *
	 * @param null $slug
	 *
	 * @return array|object|void|null
	 *
	 * @since 1.0.0
	 */
	public function get_by_slug( $slug = null ) {

		if ( empty( $slug ) ) {
			return [];
		}

		return $this->get_by( 'slug', $slug );
	}

	/**
	 * Get link by id
	 *
	 * @param int $id
	 *
	 * @return array|object|void|null
	 *
	 * @since 1.0.0
	 */
	public function get_by_id( $id = 0 ) {

		if ( empty( $id ) ) {
			return [];
		}

		return $this->get_by( 'id', $id );
	}

	/**
	 * Get links by IDs
	 *
	 * @param array $ids
	 *
	 * @return array|object|void|null
	 *
	 * @since 1.1.7
	 */
	public function get_by_ids( $ids = [] ) {

		if ( empty( $ids ) ) {
			return [];
		}

		if ( is_scalar( $ids ) ) {
			$ids = [ $ids ];
		}

		if ( ! is_array( $ids ) ) {
			return [];
		}

		$ids_str = $this->prepare_for_in_query( $ids );
		$where   = "id IN ($ids_str)";

		return $this->get_by_conditions( $where );
	}

	/**
	 * Get link by cpt_id
	 *
	 * @param int $cpt_id
	 *
	 * @return array|object|void|null
	 *
	 * @since 1.1.0
	 */
	public function get_by_cpt_id( $cpt_id = 0 ) {

		if ( empty( $cpt_id ) ) {
			return [];
		}

		return $this->get_by( 'cpt_id', $cpt_id );
	}

	/**
	 * Delete Links
	 *
	 * @param $ids
	 *
	 * @since 1.0.0
	 */
	public function delete( $ids = array() ) {

		if ( ! is_array( $ids ) ) {
			$ids = array( $ids );
		}

		if ( is_array( $ids ) && count( $ids ) > 0 ) {

			foreach ( $ids as $id ) {
				parent::delete( absint( $id ) );

				/**
				 * Take necessary cleanup steps using this hook
				 *
				 * @since 1.0.0
				 */
				do_action( 'kc_us_link_deleted', $id );
			}
		}

	}

	/**
	 * @param $url
	 * @param bool $check_domain
	 *
	 * @return array|bool
	 *
	 * @since 1.0.0
	 */
	public function is_us_link( $url, $check_domain = true ) {

		$blog_url = Helper::get_blog_url();

		if ( ! $check_domain || preg_match( '#^' . preg_quote( $blog_url ) . '#', $url ) ) {

			$uri = preg_replace( '#' . preg_quote( $blog_url ) . '#', '', $url );

			// Resolve WP installs in sub-directories
			preg_match( '#^(https?://.*?)(/.*)$#', $blog_url, $sub_directory );

			$struct = Utils::get_permalink_pre_slug_regex();

			$subdir_str = ( isset( $sub_directory[2] ) ? $sub_directory[2] : '' );

			$match_str = '#^' . $subdir_str . '(' . $struct . ')([^\?]*?)([\?].*?)?$#';

			if ( preg_match( $match_str, $uri, $match_val ) ) {

				// Match longest slug -- this is the most common
				$params = ( isset( $match_val[3] ) ? $match_val[3] : '' );

				if ( $link = $this->get_by_slug( $match_val[2] ) ) {
					return $link;
				}

				// Trim down the matched link
				$matched_link = preg_replace( '#/[^/]*?$#', '', $match_val[2], 1 );

				for ( $i = 0; ( $i < 25 ) && ! empty( $matched_link ) && ( $matched_link != $match_val[2] ); $i ++ ) {

					$new_match_str = "#^{$subdir_str}({$struct})({$matched_link})(.*?)?$#";

					$params = ( isset( $match_val[3] ) ? $match_val : '' );

					if ( $link = $this->get_by_slug( $match_val[2] ) ) {
						return $link;
					}

					// Trim down the matched link and try again
					$matched_link = preg_replace( '#/[^/]*$#', '', $match_val[2], 1 );
				}
			}
		}

		return false;
	}

	/**
	 * Prepare formdata
	 *
	 * @param array $data
	 * @param null $id
	 *
	 * @return array
	 *
	 * @since 1.1.0
	 */
	public function prepare_form_data( $data = [], $id = null ) {
		$form_data = [
			'name'              => Helper::get_data( $data, 'name', '', true ),
			'url'               => sanitize_textarea_field( Helper::get_data( $data, 'url', '' ) ),
			'slug'              => Helper::get_data( $data, 'slug', '', true ),
			'redirect_type'     => Helper::get_data( $data, 'redirection', '307', true ),
			'description'       => sanitize_textarea_field( Helper::get_data( $data, 'description', '' ) ),
			'nofollow'          => Helper::get_data( $data, 'nofollow', 0 ),
			'params_forwarding' => Helper::get_data( $data, 'params_forwarding', 0 ),
			'sponsored'         => Helper::get_data( $data, 'sponsored', 0 ),
			'track_me'          => Helper::get_data( $data, 'track_me', 0 ),
			'status'            => Helper::get_data( $data, 'status', 1 ),
			'cpt_id'            => Helper::get_data( $data, 'cpt_id', null ),
			'cpt_type'          => Helper::get_data( $data, 'cpt_type', null ),
			'expires_at'        => Helper::get_data( $data, 'expires_at', null ),
			'password'          => Helper::get_data( $data, 'password', null )
		];

		$current_user_id   = get_current_user_id();
		$current_date_time = Helper::get_current_date_time();
		// For Updaate, we want to update updated_at & updated_by_id field
		if ( ! empty( $id ) ) {
			$form_data['updated_at']    = $current_date_time;
			$form_data['updated_by_id'] = $current_user_id;
		} else {
			// For Insert, we don't need to add updated_at & updated_by_id field
			// We just need to add created_at & created_by_id field.
			$form_data['created_at']    = $current_date_time;
			$form_data['created_by_id'] = $current_user_id;
		}

		return $form_data;
	}

	/**
	 * Default Form Data
	 *
	 * @return array
	 *
	 * @since 1.1.3
	 */
	public function default_form_data() {

		return [
			'slug'              => Utils::get_valid_slug(),
			'nofollow'          => 1,
			'params_forwarding' => 0,
			'sponsored'         => 0,
			'track_me'          => 1,
		];
	}

	/**
	 * Delete clicks by cpt id
	 *
	 * @param null $cpt_id
	 *
	 * @return bool
	 *
	 * @since 1.1.0
	 */
	public function delete_by_cpt_id( $cpt_id = null ) {

		if ( empty( $cpt_id ) ) {
			return false;
		}

		return $this->delete_by( 'cpt_id', $cpt_id );
	}

	/**
	 * Create link from post
	 *
	 * @param $post
	 * @param string $slug
	 *
	 * @return bool|int
	 *
	 * @since 1.1.3
	 */
	public function create_link_from_post( $post, $slug = '' ) {

		$post = get_post( $post );

		if ( $post instanceof \WP_Post ) {

			$link_data = [
				'cpt_id'      => $post->ID,
				'url'         => get_permalink( $post->ID ),
				'name'        => addslashes( $post->post_title ),
				'description' => addslashes( $post->post_excerpt )
			];

			if ( empty( $slug ) ) {
				$slug = Utils::get_valid_slug();
			}

			$link_data['slug'] = $slug;

			$link_data = wp_parse_args( $link_data, $this->default_form_data() );

			$link_data = $this->prepare_form_data( $link_data );

			return US()->db->links->save( $link_data );
		}

		return false;
	}

	/**
	 * Create Link
	 *
	 * @param array $link_data
	 * @param string $slug
	 *
	 * @return bool|int
	 *
	 * @since 1.2.5
	 */
	public function create_link( $link_data = [], $slug = '' ) {

		if ( empty( $slug ) ) {
			$slug = Utils::get_valid_slug();
		}

		$link_data['slug'] = $slug;

		$link_data = wp_parse_args( $link_data, $this->default_form_data() );

		$link_data = $this->prepare_form_data( $link_data );

		return US()->db->links->save( $link_data );

	}


}
